@@ -76,10 +76,10 @@ typedef struct _PangoFcHbContext {
 } PangoFcHbContext;
 
 static hb_bool_t
-pango_fc_hb_font_get_glyph (hb_font_t *font, void *font_data,
-			    hb_codepoint_t unicode, hb_codepoint_t variation_selector,
-			    hb_codepoint_t *glyph,
-			    void *user_data G_GNUC_UNUSED)
+pango_fc_hb_font_get_nominal_glyph (hb_font_t *font, void *font_data,
+				    hb_codepoint_t unicode,
+				    hb_codepoint_t *glyph,
+				    void *user_data G_GNUC_UNUSED)
 {
   PangoFcHbContext *context = (PangoFcHbContext *) font_data;
   PangoFcFont *fc_font = context->fc_font;
@@ -245,7 +245,8 @@ pango_fc_get_hb_font_funcs (void)
 
   if (G_UNLIKELY (!funcs)) {
     funcs = hb_font_funcs_create ();
-    hb_font_funcs_set_glyph_func (funcs, pango_fc_hb_font_get_glyph, NULL, NULL);
+    hb_font_funcs_set_nominal_glyph_func (funcs, pango_fc_hb_font_get_nominal_glyph, NULL, NULL);
+    /* XXX we don't support variation selectors yet :(. */
     hb_font_funcs_set_glyph_h_advance_func (funcs, pango_fc_hb_font_get_glyph_advance, NULL, NULL);
     hb_font_funcs_set_glyph_v_advance_func (funcs, pango_fc_hb_font_get_glyph_advance, NULL, NULL);
     hb_font_funcs_set_glyph_h_origin_func (funcs, pango_fc_hb_font_get_glyph_h_origin, NULL, NULL);
@@ -342,6 +343,9 @@ _pango_fc_shape (PangoFont           *font,
   hb_buffer_set_direction (hb_buffer, hb_direction);
   hb_buffer_set_script (hb_buffer, hb_glib_script_to_script (analysis->script));
   hb_buffer_set_language (hb_buffer, hb_language_from_string (pango_language_to_string (analysis->language), -1));
+#if HB_VERSION_ATLEAST(1,0,3)
+  hb_buffer_set_cluster_level (hb_buffer, HB_BUFFER_CLUSTER_LEVEL_MONOTONE_CHARACTERS);
+#endif
   hb_buffer_set_flags (hb_buffer,
 		       (item_offset == 0 ? HB_BUFFER_FLAG_BOT : 0) |
 		       (item_offset + item_length == paragraph_length ? HB_BUFFER_FLAG_EOT : 0));
@@ -390,11 +394,7 @@ _pango_fc_shape (PangoFont           *font,
 		    len = -1;
 
 		  if (hb_feature_from_string (feat, len, &features[num_features]))
-		  {
 		    num_features++;
-		    features[num_features].start = 0;
-		    features[num_features].end = -1;
-		  }
 
 		  if (end == NULL)
 		    break;
