@@ -21,9 +21,12 @@
 
 #include <glib.h>
 #include <string.h>
-#include <unistd.h>
 #include <locale.h>
 
+#ifndef G_OS_WIN32
+#include <unistd.h>
+#endif
+
 #include <pango/pangocairo.h>
 #include "test-common.h"
 
